import pandas as pd
import numpy as np
from tools import preprocessor, file_reader, explainer
import paths as pt
import os
import csv
import joblib
from pathlib import Path

def main():
    model = file_reader.read_joblib(pt.COMPLETE_XGB_DIR,
                                    'complete_xgboost.joblib')
    input_data = {"Gender": [0],
                "BirthYear": [46],
                "LoanPeriod": [360],
                "Ats": ["093307,222718,181210"]}
    new_data_df = pd.DataFrame.from_dict(input_data)
    new_data_df['NumberAts'] = len(new_data_df['Ats'][0].split(","))
    df = preprocessor.split_cat_columns(new_data_df, col_to_split='Ats',
                                        tag='Ats',
                                        resolution=10)
    
    cols_ats = [str(i)+'Ats' for i in range(1, 10+1)]
    header_list = ['Gender', 'BirthYear', 'Cluster',
                'LoanPeriod', 'NumberAts'] + cols_ats
    df = df.reindex(columns=header_list)
    df = df.fillna('0')
    
    df['Cluster'] = 14
    df['Cluster'] = pd.to_numeric(df['Cluster'])
    
    for i in range(1, 10+1):
        path = Path.joinpath(pt.PROCESSED_DATA_DIR, 'embeddings')
        embedding = file_reader.read_embedding(path, f'complete_{i}Ats.csv')
        column = f'{i}Ats'
        df[column] = df[column].replace(to_replace=embedding)
        df[column] = pd.to_numeric(df[column])
    
    prediction = model.predict(df)
    probability = model.predict_proba(df).max()
    _, shap_values = explainer.get_shap_tree_explainer(model, X_test=df)
    shap_values_flat = [round(float(val), 3) for val in shap_values[0]]
    shap_values_dict = dict(zip(df.columns, shap_values_flat))
    
    print(f"Predicted {int(prediction[0])} with probability {round(float(probability), 3)*100}%")
    
    for item, amount in shap_values_dict.items():
        print("{} ({})".format(item, amount))
        
if __name__ == "__main__":
    main()